#    This file is part of MK.
#
#    MK is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MK is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os
import types
import mk
import mk.meta

from autodep import cpp_scan

class cc (object):
    __metaclass__ = mk.meta.autoprop

    def __init__ (self):
        self.cc = 'cc'
        self._debug = True
        self._optimize = False
        self._profile = False
        self._warning = True
        self._pic = False
        self._cpu = ''
        self._tune = ''
        self.extra_flags = []
        self._includes = []
        self._defines = {}
        self._update_flags ();

    # Append include path(s).
    def include (self, path):
        if isinstance (path, types.ListType):
            self._includes.extend ([os.path.abspath (x) for x in path])
        else:
            self._includes.extend ([os.path.abspath (path)])

    def _get_includes (self):
        return self._includes

    # Define a preprocessor macro.
    def define (self, df, value = None):
        if isinstance (df, types.DictType):
            self._defines.update (df)
        else:
            self._defines [df] = value;

    def _get_defines (self):
        return self._defines

    # Update the _flags member, after a member changes.
    def _update_flags (self):
        self._flags = ['-c']
        if self._debug:
            self._flags.append ('-g')
        if self._optimize:
            self._flags.append ('-O%s' % self._optimize)
        if self._profile:
            self._flags.append ('-p')

    # Debug attribute.
    def _set_debug (self, debug):
        self._debug = debug
        self._update_flags ()

    def _get_debug (self):
        return self._debug

    # Optimization level attribute.
    def _set_optimize (self, opt):
        self._optimize = opt
        self._update_flags ()

    def _get_optimize (self):
        return self._optimize

    # Profile attribute.
    def _set_profile (self, prof):
        self_._profile = prof
        self._update_flags ()

    def _get_profile (self):
        return self._profile

    # Warning attribute.
    def _set_warning (self, warn):
        self._warning = warn
        self._update_flags ()

    def _get_warning (self):
        return self._warning

    # Position Independent Code attribute.
    def _set_pic (self, pic):
        self._pic = pic
        self._update_flags ()

    def _get_pic (self):
        return self._pic

    # Host cpu attribute.  
    def _set_cpu (self, cpu):
        self._cpu = cpu
        self._update_flags ()

    def _get_cpu (self):
        return self._cpu

    # Host cpu tune attribute.  
    def _set_tune (self, tune):
        self._tune = tune
        self._update_flags ()

    def _get_tune (self):
        return self._tune

    # Construct include path options.
    def _include_opt (self):
        return ['-I%s' % mk.shortname (x) for x in self._includes]

    # Construct preprocessor define options.
    def _define_opt (self):
        opt = []
        for x, y in self._defines.iteritems ():
            if y != None:
                opt.append ('-D%s=%s' % (x, y))
            else:
                opt.append ('-D%s' % x);
        return opt

    # Construct option to name the output file.
    def _output_opt (self, name):
        return ['-o', name]

    # Compile the sources.  
    def compile (self, target):
        cmd = [self.cc]
        cmd.extend (self._flags)
        cmd.extend (self.extra_flags)
        cmd.extend (self._include_opt ())
        cmd.extend (self._define_opt ())
        cmd.extend (self._output_opt (mk.shortname (target.name)))
        cmd.append (mk.shortname (target.source.name))
        return cmd

    # Compile source to Position Independent Code.  
    def compile_pic (self, target):
        return self.compile (target)

    # Use the generic CPP directives scanner for automatic
    # dependencies.
    def autodep (self, nd, deps):
        cpp_scan.autodep (nd.name, self._includes, deps)

def get_class ():
    return cc

# Local variables:
# mode: python
# indent-tabs-mode: nil
# arch-tag: 2a037fe7-dc9a-405e-96c4-f62207e689b1
# End:
