#    This file is part of MK.
#
#    MK is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MK is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import mk
from cc import cc

_gcc_cpu_names = { 'pentiumii' : 'pentium2', 'pentiumiii' : 'pentium3' }

def _gcc_cpu_name (name):
    if _gcc_cpu_names.has_key (name):
        return _gcc_cpu_names (name)
    else:
        return name

class gcc (cc):
    def __init__ (self):
        super (gcc, self).__init__ ()
        self.cc = 'gcc'

    # Update the _flags member, after a member changes.
    def _update_flags (self):
        super (gcc, self)._update_flags ()
        if self._warning:
            self._flags.extend (['-Wall', '-W'])
        if self._cpu:
            self._flags.append ('-march=%s' % _gcc_cpu_name (self._cpu))
        if self._tune:
            self._flags.append ('-mtune=%s' % _gcc_cpu_name (self._cpu))

    def compile_pic (self, target):
        cmd = [self.cc, '-fPIC']
        cmd.extend (self._flags)
        cmd.extend (self.extra_flags)
        cmd.extend (self._include_opt ())
        cmd.extend (self._define_opt ())
        cmd.extend (self._output_opt (mk.shortname (target.name)))
        cmd.append (mk.shortname (target.source.name))
        return cmd

def get_class ():
    return gcc

# Local variables:
# mode: python
# indent-tabs-mode: nil
# arch-tag: 6cc50ff6-cb7f-4ee1-a742-3a538fd520ad
# End:
