#    This file is part of MK.
#
#    MK is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    MK is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Foobar; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os.path
import mk
import mk.meta

class ld (object):
    __metaclass__ = mk.meta.autoprop

    def __init__ (self):
        self.ld = 'ld'
        self._debug = True
        self._profile = False
        self.extra_flags = ''
        self._update_flags ()

    def _update_flags (self):
        self._flags = []
        if self._debug:
            self._flags.append ('-g')
        if self._profile:
            self._flags.append ('-p')

    # Debug flag.
    def _set_debug (self, dbg):
        self._debug = dbg
        self._update_flags ()

    def _get_debug (self):
        return self._debug

    # Profile flag.
    def _set_profile (self, prof):
        self._profile = prof
        self._update_flags ()

    def _get_profile (self):
        return self._profile ()

    # Library reference.
    def _libref (self, name):
        if name.endswith ('.a'):
            nm = name [:-2]
        else:
            i = name.rfind ('.so');
            if i != -1:
                nm = name [0:i]
            else:
                return [name]
        dir = os.path.dirname (nm)
        base = os.path.basename (nm)
        if base.startswith ('lib'):
            if len (dir):
                return ['-L' + mk.shortname (dir), '-l' + base [3:]]
            else:
                return ['-l' + base [3:]]
        else:
            return [name]

    # Name the output file:
    def _output_opt (self, name):
        return ['-o', name]

    # Generate relocatable file option.
    def _reloc_opt (self):
        return ['-r']
    
    # Generate shared library option.
    def _shared_opt (self):
        return ['-Bshareable']
    
    # Build an executable file.  
    def link_program (self, target):
        cmd = [self.ld]
        cmd.extend (self._flags)
        cmd.extend (self.extra_flags)
        cmd.extend (self._output_opt (mk.shortname (target.name)))
        for obj in target.objects:
            cmd.append (mk.shortname (obj.name))
        for lib in target.libs:
            cmd.extend (self._libref (lib.name))
        for lib in target.xlibs:
            cmd.extend (self._libref (lib))
        return cmd

    # Build a relocatable object file.  
    def link_relocatable (self, target):
        cmd = [self.ld]
        cmd.extend (self._reloc_opt ())
        cmd.extend (self._flags)
        cmd.extend (self.extra_flags)
        cmd.extend (self._output_opt (mk.shortname (target.name)))
        for obj in target.objects:
            cmd.append (shortname (obj.name))
        for lib in target.libs:
            cmd.extend (self._libref (lib.name))
        for lib in target.xlibs:
            cmd.extend (self._libref (lib))
        return cmd

    # Build a shared library.  
    def link_shared_lib (self, target):
        cmd = [self.ld]
        cmd.extend (self._shared_opt ())
        cmd.extend (self._flags)
        cmd.extend (self.extra_flags)
        cmd.extend (self._output_opt (mk.shortname (target.name)))
        for obj in target.objects:
            cmd.append (shortname (obj.name))
        for lib in target.libs:
            cmd.extend (self._libref (lib.name))
        for lib in target.xlibs:
            cmd.extend (self._libref (lib))
        return cmd

def get_class ():
    return ld

# Local variables:
# mode: python
# indent-tabs-mode: nil
# arch-tag: 665e8b28-cf82-4f7b-b744-0f5be7e5310f
# End:
